using System;
using Microsoft.SPOT.Hardware;
using System.Threading;
using System.Collections;


/* 
 * 
 * Copyright David Zimmer 2013 http://sandsprite.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

namespace Sandsprite.Toolbox.Netduino
{
    public class cCol
    {
        public InterruptPort ip = null;
        public int pinID = 0;
        public int index = 0;

        public cCol(int i, Cpu.Pin pin, NativeEventHandler neh)
        {
            ip = new InterruptPort(pin, true, Port.ResistorMode.PullUp, Port.InterruptMode.InterruptEdgeLevelLow);
            ip.OnInterrupt += neh;
            pinID = (int)pin;
            index = i;
        }
    }

    public class MatrixKeyPad
    {
        
        public delegate void KeyPressEvent(int row, int col, int KeyCode, DateTime stamp);
        public DateTime lastPress = DateTime.Now;
        public uint Debounce = 0;

        private bool Processing = false;
        private KeyPressEvent KeyHandler;
        private OutputPort[] Rows;
        private Hashtable Cols = new Hashtable();

        public MatrixKeyPad(Cpu.Pin[] RowPins, Cpu.Pin[] ColPins, KeyPressEvent kpe)
        {
            KeyHandler = kpe;

            this.Rows = new OutputPort[RowPins.Length];
            for (var i = 0; i < RowPins.Length; ++i)
            {
                this.Rows[i] = new OutputPort(RowPins[i], false);   
            }

            for (int i = 0; i < ColPins.Length; ++i)
            {
                cCol c = new cCol(i, ColPins[i], KeyPadCol_OnInterrupt);
                Cols.Add("pin:"+ColPins[i].ToString(), c); 
            }

        }

        void KeyPadCol_OnInterrupt(uint ColPinId, uint State, DateTime time)
        {
            
            int row, keyindex = -1;
            bool fire = false;

            if (Processing) return;

            cCol col = (cCol)Cols["pin:" + ColPinId.ToString()];
            if (col == null) return;

            Processing = true;
            EnablelInterrupts(false);
            SetRowsHigh(true); 

            for (row = 0; row < Rows.Length; row++)
            {
                Rows[row].Write(false);   //set this row low and 
                if (isLow(col.ip))        //look to see if it pulled our triggered column low..
                {
                    while ( isLow(col.ip) ) { ; }                    //wait until keyup
                    keyindex = (int)(row * Cols.Count + col.index);  //calculate the key index
                    break;
                } 
            }

            if (keyindex != -1)
            {
                fire = true;
                if (Debounce > 0)
                {
                    TimeSpan span = time.Subtract(lastPress);
                    if (span.Ticks < Debounce) fire = false;
                }
                lastPress = time;
                if(fire) KeyHandler(row, col.index, keyindex, DateTime.Now);
            }

            SetRowsHigh(false);
            EnablelInterrupts(true);
            Processing = false;
             
        }

        private bool isLow(InterruptPort port) { return !port.Read(); }

        private void SetRowsHigh(bool setHigh)
        {
            for (var i = 0; i < this.Rows.Length; ++i) Rows[i].Write(setHigh);
        }

        private void EnablelInterrupts(bool enable)
        {
            if(enable) foreach (cCol col in Cols.Values )  col.ip.EnableInterrupt();
            else foreach (cCol col in Cols.Values )  col.ip.DisableInterrupt();
        }

    }
}
