using System;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using SecretLabs.NETMF.Hardware;
using SecretLabs.NETMF.Hardware.Netduino;
using Sandsprite.Toolbox.Netduino;

/*
 * Copyright 2013 David Zimmer (http://sandsprite.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
namespace Matrix_KeyPad
{
    public class Program
    {
        static string[] key_input;
        static string passwd = "123";
        static int blinkDelay = 200;
        static int curIndex = 0;
        static MatrixKeyPad kb;

        static OutputPort ok_led = new OutputPort(Pins.GPIO_PIN_D13, false);
        static OutputPort fail_led = new OutputPort(Pins.GPIO_PIN_D12, false);

        static void BlinkLED(OutputPort led)
        {
            for (int i = 1; i <= 3; i++)
            {
                led.Write(true);
                Thread.Sleep(blinkDelay);
                led.Write(false);
                Thread.Sleep(blinkDelay);
            }
        }

        static void CheckLogin()
        {
            string userPass = "";
            for(int i=0;i<key_input.Length;i++) userPass+=key_input[i];

            if (userPass == passwd)
            {
                BlinkLED(ok_led);
            }
            else
            {
                Debug.Print("Failed login: " + userPass);
                BlinkLED(fail_led);
            }

        }

        public static void Main()
        {

            //this example uses the following 4x4 keypad: 
            //  http://www.amazon.com/gp/product/B008A30NW4/ref=oh_details_o04_s00_i00?ie=UTF8&psc=1
            //it is wired with from left D0 to right D7 as you look at the keypad
            //a green LED with resistor is on D13, and a red LED tied into same resistor to ground on D12
            //
            //behavior: it will accept passwd.length keystrokes and then check received values against the hardcoded
            //          pass. if they match blink green led, if not blink red. key entries is reset after attempt.

            key_input = new string[passwd.Length];

            Cpu.Pin[] RowPins = { Pins.GPIO_PIN_D0, Pins.GPIO_PIN_D1, Pins.GPIO_PIN_D2, Pins.GPIO_PIN_D3 };
            Cpu.Pin[] ColPins = { Pins.GPIO_PIN_D4, Pins.GPIO_PIN_D5, Pins.GPIO_PIN_D6, Pins.GPIO_PIN_D7 };
            kb = new MatrixKeyPad(RowPins, ColPins, new MatrixKeyPad.KeyPressEvent(kb_KeyPress));
            Thread.Sleep(Timeout.Infinite);

        }

        static void kb_KeyPress(int row, int col, int KeyCode, DateTime time)
        {

            //translate the raw keypad matrix index to our keypads key markings
            string[] xlate = new string[] { "1", "2", "3", "A", "4", "5", "6", "B", "7", "8", "9", "C", "*", "0", "#", "D" };

            Debug.Print("Key: " + xlate[KeyCode] + " Row: " + row + " Col: " + col + " Time: " + time.ToString() );

            key_input[curIndex] = xlate[KeyCode];
            curIndex++;

            if (curIndex == key_input.Length)
            {
                CheckLogin();
                curIndex = 0;
                //Array.Clear(key_input, 0, key_input.Length);
            } 
 
        }
    }
}
