﻿using System;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using SecretLabs.NETMF.Hardware;
using SecretLabs.NETMF.Hardware.Netduino;

namespace Spoofer
{
    public class Program
    {
       
        static InterruptPort Failed;
        static InterruptPort Success;
        static DateTime lastFail = DateTime.Now;
        static DateTime startTime = DateTime.Now;
        static string curLogin = "";
        static OutputPort[] row_port = new OutputPort[4];
        static OutputPort[] col_port = new OutputPort[4];
        static string[] test = new string[1000];
        static int testIndex = 0;
        static bool waiting = false;

        static void BuildTests()
        {
            //Generates all possible 3 button numeric key combinations 000, 001 etc..
            //a better practive would be to generate on the fly and not waste all this memory
            //but by the same token its good to test the available memory of the microcontroller to see
            //just what it can and cant do and how well managed the .NET micro memory is.
            int i = 0;

            for (int a = 0; a < 10; a++)
            {
                for (int b = 0; b < 10; b++)
                {
                    for (int c = 0; c < 10; c++)
                    {
                        string x = a.ToString() + b.ToString() + c.ToString();
                        test[i] = x;
                        i++;
                    }
                }
            }
        }

        static string IDToName(Cpu.Pin id)
        {
            Cpu.Pin[] DigitalPins = { Pins.GPIO_PIN_D0, Pins.GPIO_PIN_D1, Pins.GPIO_PIN_D2, Pins.GPIO_PIN_D3,Pins.GPIO_PIN_D4, Pins.GPIO_PIN_D5, Pins.GPIO_PIN_D6, Pins.GPIO_PIN_D7 };
            string[] n = new string[]{"GPIO_PIN_D0", "GPIO_PIN_D1", "GPIO_PIN_D2", "GPIO_PIN_D3", "GPIO_PIN_D4", "GPIO_PIN_D5", "GPIO_PIN_D6", "GPIO_PIN_D7"};
            for (int i = 0; i < DigitalPins.Length; i++) if (id == DigitalPins[i]) return n[i];
            return "Unknown pin id: " + id.ToString();
        }

        static void SendKey(char x)
        {
            
            string xlate = "123A" + "456B" + "789C" + "*0#D";
            int i = xlate.IndexOf(x);
            if (i < 0) return;

            int row = i / 4;
            int col = i - (row * 4);

            OutputPort pRow = row_port[row];
            OutputPort pCol = col_port[col];

            //see captured debug output at bottom of file..
            //Debug.Print("Char: " + x.ToString() + " Row: " + row + " Col:" + col + "   " + IDToName(pRow.Id) + " / " + IDToName(pCol.Id));

            pRow.Write(false); //set targets low to activate relays..
            pCol.Write(false);

            Thread.Sleep(50);

            pCol.Write(true); //disable relays again..
            pRow.Write(true);

            Thread.Sleep(50);

        }

        public static void Main()
        {

            //we will assume manual sync..(fresh password ready state)
            //InputPort sync = new InputPort(Pins.GPIO_PIN_D10, true, Port.ResistorMode.PullUp);

            Cpu.Pin[] RowPins = { Pins.GPIO_PIN_D0, Pins.GPIO_PIN_D1, Pins.GPIO_PIN_D2, Pins.GPIO_PIN_D3 };
            Cpu.Pin[] ColPins = { Pins.GPIO_PIN_D4, Pins.GPIO_PIN_D5, Pins.GPIO_PIN_D6, Pins.GPIO_PIN_D7 };

            for (int i = 0; i < RowPins.Length; i++)
            {
                row_port[i] = new OutputPort(RowPins[i], true);
                col_port[i] = new OutputPort(ColPins[i], true);
            }


            /*
            //test sequence generator...
            string xlate = "123A" + "456B" + "789C" + "*0#D";
            char[] x = xlate.ToCharArray();
            for (int i = 0; i < xlate.Length; i++)
            {
                SendKey(x[i]);
                Thread.Sleep(1000);
            } 

            Thread.Sleep(Timeout.Infinite);
             */

            //failed port has a mini grabber hooked to the login failure red led
            //success port has a mini grabber hooked to the login success green led
            Failed = new InterruptPort(Pins.GPIO_PIN_D10, true, Port.ResistorMode.PullUp, Port.InterruptMode.InterruptEdgeLevelLow);
            Success = new InterruptPort(Pins.GPIO_PIN_D11, true, Port.ResistorMode.PullUp , Port.InterruptMode.InterruptEdgeLevelHigh);
            Failed.OnInterrupt += new NativeEventHandler(Failed_OnInterrupt);
            Success.OnInterrupt += new NativeEventHandler(Success_OnInterrupt);
            Failed.EnableInterrupt();
            Success.EnableInterrupt();

            BuildTests();
            for (int testIndex = 0; testIndex < test.Length; testIndex++)
            {
                curLogin = test[testIndex];
                Debug.Print("Trying: " + curLogin);
                waiting = true;
                char[] c = test[testIndex].ToCharArray();
                for (int i = 0; i < c.Length; i++)
                {
                    if(!waiting){ //unexpected are we out of sequence resync..
                        Debug.Print("Resyncing oops...");
                        //Resync();
                        break;
                    }
                    SendKey(c[i]);
                }
                int t = 0;
                while (waiting)
                {
                    t++;
                    if (t % 1000 == 0)
                    {
                        TimeSpan span = DateTime.Now.Subtract(lastFail);
                        if (span.Ticks > 1000 * 60 * 60)
                        {
                            Debug.Print("Wait exceeded, maybe lost keypress trying to resync..");
                            //Resync();
                            break;
                        }
                    }
                    Thread.Sleep(5);
                     
                }
                Thread.Sleep(1800);
            }

            Thread.Sleep(Timeout.Infinite);
        }

        /* nice idea but not worth the labor to debug it, 
         * working well enough as is for an experiment...
         
        static void Resync()
        {
            lastFail = DateTime.Now;
            while (waiting)
            {
                SendKey('*');
                Thread.Sleep(1000);
                TimeSpan span = DateTime.Now.Subtract(lastFail);
                if (span.Ticks > 1000 * 60 * 60)
                {
                    Debug.Print("Reader hung?? Double time out...");
                    break;
                }
            }

        }*/

        static void Success_OnInterrupt(uint ColPinId, uint State, DateTime time)
        {

            TimeSpan span = time.Subtract(lastFail);
            //if (span.Seconds < 1) return; //why is this triggering after the fail? 
            
            Failed.DisableInterrupt();
            Success.DisableInterrupt();

            span = time.Subtract(startTime);
            string msg = "LOGIN FOUND! " + curLogin;
            if(span.Minutes==0) msg+= " Seconds: " + span.Seconds.ToString();
            else msg += " Minutes: " + span.Minutes.ToString(); 

            Debug.Print(msg);
              
        }


        static void Failed_OnInterrupt(uint ColPinId, uint State, DateTime time)
        {
            Failed.DisableInterrupt();
            

            TimeSpan span = time.Subtract(lastFail);
            if (span.Seconds > 2)
            {
                Debug.Print("Failed Login! ");// + time.ToString());
                lastFail = time;
                waiting = false; //failure message received..
            }

            
            //Failed.ClearInterrupt();
            Failed.EnableInterrupt();
            
        }

        /*
            Char: 1 Row: 0 Col:0   GPIO_PIN_D0 / GPIO_PIN_D4
            Char: 2 Row: 0 Col:1   GPIO_PIN_D0 / GPIO_PIN_D5
            Char: 3 Row: 0 Col:2   GPIO_PIN_D0 / GPIO_PIN_D6
            Char: A Row: 0 Col:3   GPIO_PIN_D0 / GPIO_PIN_D7
            Char: 4 Row: 1 Col:0   GPIO_PIN_D1 / GPIO_PIN_D4
            Char: 5 Row: 1 Col:1   GPIO_PIN_D1 / GPIO_PIN_D5
            Char: 6 Row: 1 Col:2   GPIO_PIN_D1 / GPIO_PIN_D6
            Char: B Row: 1 Col:3   GPIO_PIN_D1 / GPIO_PIN_D7
            Char: 7 Row: 2 Col:0   GPIO_PIN_D2 / GPIO_PIN_D4
            Char: 8 Row: 2 Col:1   GPIO_PIN_D2 / GPIO_PIN_D5
            Char: 9 Row: 2 Col:2   GPIO_PIN_D2 / GPIO_PIN_D6
            Char: C Row: 2 Col:3   GPIO_PIN_D2 / GPIO_PIN_D7
            Char: * Row: 3 Col:0   GPIO_PIN_D3 / GPIO_PIN_D4
            Char: 0 Row: 3 Col:1   GPIO_PIN_D3 / GPIO_PIN_D5
            Char: # Row: 3 Col:2   GPIO_PIN_D3 / GPIO_PIN_D6
            Char: D Row: 3 Col:3   GPIO_PIN_D3 / GPIO_PIN_D7
         */ 
        
    }
}
